/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: Graph.java,v 1.10 2007/05/19 02:45:24 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph;

import java.util.Collection;
import java.util.List;

import org.xmlpull.infoset.XmlElement;

import edu.indiana.extreme.xbaya.graph.gui.GraphGUI;

/**
 * @author Satoshi Shirasuna
 */
public interface Graph extends GraphPiece {

    /**
     * Returns the gui.
     * 
     * @return the gui
     */
    public GraphGUI getGUI();

    /**
     * Returns the ID of this graph.
     * 
     * @return The ID of this graph
     */
    public String getID();

    /**
     * Sets the name of this graph.
     * 
     * @param name
     *            The name to set
     */
    public void setName(String name);

    /**
     * Returns the name of this graph.
     * 
     * @return The name
     */
    public String getName();

    /**
     * Returns the description of this graph.
     * 
     * @return The description.
     */
    public String getDescription();

    /**
     * Sets the description of this graph.
     * 
     * @param description
     *            The description to set.
     */
    public void setDescription(String description);

    /**
     * Returns the list of the nodes of this graph.
     * 
     * It's a list because the order of input nodes matters.
     * 
     * @return the list of the nodes
     */
    public List<? extends Node> getNodes();

    /**
     * Returns the list of the ports of this graph.
     * 
     * @return the list of the ports
     */
    public Collection<? extends Port> getPorts();

    /**
     * Returns the list of edges of this graph.
     * 
     * @return the list of edges
     */
    public Collection<? extends Edge> getEdges();

    /**
     * Removes a specified node from this graph.
     * 
     * @param node
     *            the node to delete
     * @throws GraphException
     *             If the specified node does not exist in the graph
     */
    public void removeNode(Node node) throws GraphException;

    /**
     * Returns a node with a specified ID.
     * 
     * @param nodeID
     *            The specified ID.
     * @return The node with the specified ID if exists; null otherwise
     */
    public Node getNode(String nodeID);

    /**
     * Returns a port with a specified ID.
     * 
     * @param portID
     *            The specified ID.
     * @return The port with the specified ID if exists; null otherwise
     */
    public Port getPort(String portID);

    /**
     * Adds an edge between two ports specified. If the edge already exists
     * between two ports, it returns null without creating a new one.
     * 
     * @param fromPort
     *            The port the edge is connected from
     * @param toPort
     *            The port the edge is connected to
     * @return The edge added
     * @throws GraphException
     */
    public Edge addEdge(Port fromPort, Port toPort) throws GraphException;

    /**
     * Removes a specified edge.
     * 
     * @param edge
     *            The edge to remove.
     * @throws GraphException
     */
    public void removeEdge(Edge edge) throws GraphException;

    /**
     * Removes an Edge between two specified ports.
     * 
     * @param fromPort
     * @param toPort
     * @throws GraphException
     */
    public void removeEdge(Port fromPort, Port toPort) throws GraphException;

    /**
     * Checks if an Edge exist between two specified ports.
     * 
     * @param fromPort
     * @param toPort
     * @return true if an Edge exists; false otherwise;
     */
    public boolean containsEdge(Port fromPort, Port toPort);

    /**
     * Imports a Graph to this Graph.
     * 
     * @param graph
     *            the Graph to import
     * @throws GraphException
     */
    public void importGraph(Graph graph) throws GraphException;

    /**
     * @return The graph XML
     */
    public XmlElement toXML();
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
